/*!\file openCV.cpp
 * \brief intro OpenCV, transformer HSV et garder le vert.
 *
 * \author Farès BELHADJ, amsi@ai.univ-paris8.fr
 * \date September 13 2018
 */

#include <opencv2/opencv.hpp>
#include <iostream>

using namespace cv;
using namespace std;

/* minimum et maximum entre trois valeurs */
#define MIN3(a, b, c) ( (a) < (b) ? ( (a) < (c) ? (a) : ( (b) < (c) ? (b) : (c) ) ) : ( (b) < (c) ? (b) : (c) ) )
#define MAX3(a, b, c) ( (a) > (b) ? ( (a) > (c) ? (a) : ( (b) > (c) ? (b) : (c) ) ) : ( (b) > (c) ? (b) : (c) ) )

/* conversion de rgb vers hsv */
static void rgb2hsv(float r, float g, float b, float * h, float * s, float * v) {
  float min = MIN3(r, g, b), max = MAX3(r, g, b);
  *v = max;
  *s = (max != 0.0f) ? ((max - min) / max) : 0.0f;
  if(*s == 0.0f)
    *h = 1.0f;
  else {
    float delta = 1.0f / (max - min);
    if(r == max)
      *h = delta * (g - b) ;
    else if(g == max)
      *h = 2.0f + delta * (b - r);
    else //if(b == max)
      *h = 4.0f + delta * (r - g);
    *h = (*h < 0.0f) ? (*h / 6.0f + 1.0f) : (*h / 6.0f);
  }
}

/* parcoourir l'image et ne garder que des couleurs "vertes" : 
 *  25% < H < 55% et S > 30% */
static void garderLeVert(Mat& img) {
  int i, j, ii, jj;
  float r, g, b, h, s, v;
  uint8_t * dd = img.data;
  assert(img.channels() == 3);
  for(i = 0; i < img.rows; ++i) {
    ii = i * img.step;
    for(j = 0; j < img.cols; ++j) {
      jj = ii + j * img.channels();
      r = dd[jj + 2] / 255.0f;
      g = dd[jj + 1] / 255.0f;
      b = dd[jj + 0] / 255.0f;
      rgb2hsv(r, g, b, &h, &s, &v);
      if(h >= 0.25f && h <= 0.55f && s > 0.3f)
	dd[jj + 0] = dd[jj + 1] = dd[jj + 2] = sqrt(s) * 255.0f;
      else
	dd[jj + 0] = dd[jj + 1] = dd[jj + 2] = 0;
    }
  }
}

int main(int argc, char ** argv) {
  /* essayer d'ouvrir la seconde caméra */
  VideoCapture cap(1);
  /* si echec */
  if(!cap.isOpened()) {
    /* essayer d'ouvrir n'importe quelle caméra */
    cap.open(CV_CAP_ANY);
    /* si echec, quitte */
    if(!cap.isOpened())
      return 1;
  }
  /* on récupère et imprime les dimensions de la vidéo */
  Size s(cap.get(CV_CAP_PROP_FRAME_WIDTH), cap.get(CV_CAP_PROP_FRAME_HEIGHT));
  cerr << "dimensions du flux " << s.width << "x" << s.height << endl;
  /* créer une fenêtre */
  namedWindow("Garder le vert", WINDOW_AUTOSIZE);
  /* une image */
  Mat frame;
  /* boucle principale */
  for(;;) {
    /* récupérer la frame courante */
    cap >> frame;
    /* ne garder que le vert dans l'image */
    garderLeVert(frame);
    /* afficher les deux images */
    imshow("Garder le vert", frame);
    /* attendre 10ms pour capture clavier, si echap alors break */
    if((waitKey(10) & 0xFF) == 27)
      break;
  }
  return 0;
}

